package sun.management;

import java.io.InvalidObjectException;
import java.lang.management.MemoryUsage;
import java.util.Map;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.OpenDataException;
import java.security.AccessController;
import java.lang.reflect.Field;
import java.security.PrivilegedAction;
import javax.management.openmbean.CompositeData;
import javax.management.openmbean.CompositeType;
import javax.management.openmbean.OpenType;
import com.sun.management.GcInfo;

public class GcInfoCompositeData extends LazyCompositeData
{
    private final GcInfo info;
    private final GcInfoBuilder builder;
    private final Object[] gcExtItemValues;
    private static final String ID = "id";
    private static final String START_TIME = "startTime";
    private static final String END_TIME = "endTime";
    private static final String DURATION = "duration";
    private static final String MEMORY_USAGE_BEFORE_GC = "memoryUsageBeforeGc";
    private static final String MEMORY_USAGE_AFTER_GC = "memoryUsageAfterGc";
    private static final String[] baseGcInfoItemNames;
    private static MappedMXBeanType memoryUsageMapType;
    private static OpenType[] baseGcInfoItemTypes;
    private static CompositeType baseGcInfoCompositeType;
    private static final long serialVersionUID = -5716428894085882742L;
    
    public GcInfoCompositeData(GcInfo info, GcInfoBuilder builder, Object[] gcExtItemValues) {
        this.info = info;
        this.builder = builder;
        this.gcExtItemValues = gcExtItemValues;
    }
    
    public GcInfo getGcInfo() {
        return this.info;
    }
    
    public static CompositeData toCompositeData(final GcInfo gcInfo) {
        return new GcInfoCompositeData(gcInfo, (GcInfoBuilder)AccessController.doPrivileged(new PrivilegedAction<GcInfoBuilder>() {
            @Override
            public GcInfoBuilder run() {
                try {
                    Field declaredField = Class.forName("com.sun.management.GcInfo").getDeclaredField("builder");
                    declaredField.setAccessible(true);
                    return (GcInfoBuilder)declaredField.get(gcInfo);
                }
                catch (ClassNotFoundException | NoSuchFieldException | IllegalAccessException ex) {
                    return null;
                }
            }
        }), (Object[])AccessController.doPrivileged(new PrivilegedAction<Object[]>() {
            @Override
            public Object[] run() {
                try {
                    Field declaredField = Class.forName("com.sun.management.GcInfo").getDeclaredField("extAttributes");
                    declaredField.setAccessible(true);
                    return (Object[])(Object[])declaredField.get(gcInfo);
                }
                catch (ClassNotFoundException | NoSuchFieldException | IllegalAccessException ex) {
                    return null;
                }
            }
        })).getCompositeData();
    }
    
    @Override
    protected CompositeData getCompositeData() {
        Object[] array;
        try {
            array = new Object[] { new Long(this.info.getId()), new Long(this.info.getStartTime()), new Long(this.info.getEndTime()), new Long(this.info.getDuration()), GcInfoCompositeData.memoryUsageMapType.toOpenTypeData(this.info.getMemoryUsageBeforeGc()), GcInfoCompositeData.memoryUsageMapType.toOpenTypeData(this.info.getMemoryUsageAfterGc()) };
        }
        catch (OpenDataException detailMessage) {
            throw new AssertionError(detailMessage);
        }
        int gcExtItemCount = this.builder.getGcExtItemCount();
        if (gcExtItemCount == 0 && this.gcExtItemValues != null && this.gcExtItemValues.length != 0) {
            throw new AssertionError("Unexpected Gc Extension Item Values");
        }
        if (gcExtItemCount > 0 && (this.gcExtItemValues == null || gcExtItemCount != this.gcExtItemValues.length)) {
            throw new AssertionError("Unmatched Gc Extension Item Values");
        }
        Object[] itemValues = new Object[array.length + gcExtItemCount];
        System.arraycopy(array, 0, itemValues, 0, array.length);
        if (gcExtItemCount > 0) {
            System.arraycopy(this.gcExtItemValues, 0, itemValues, array.length, gcExtItemCount);
        }
        try {
            return new CompositeDataSupport(this.builder.getGcInfoCompositeType(), this.builder.getItemNames(), itemValues);
        }
        catch (OpenDataException detailMessage2) {
            throw new AssertionError(detailMessage2);
        }
    }
    
    static String[] getBaseGcInfoItemNames() {
        return GcInfoCompositeData.baseGcInfoItemNames;
    }
    
    static synchronized OpenType[] getBaseGcInfoItemTypes() {
        if (GcInfoCompositeData.baseGcInfoItemTypes == null) {
            OpenType<Object> openType = GcInfoCompositeData.memoryUsageMapType.getOpenType();
            GcInfoCompositeData.baseGcInfoItemTypes = new OpenType[] { SimpleType.LONG, SimpleType.LONG, SimpleType.LONG, SimpleType.LONG, openType, openType };
        }
        return GcInfoCompositeData.baseGcInfoItemTypes;
    }
    
    public static long getId(CompositeData compositeData) {
        return LazyCompositeData.getLong(compositeData, "id");
    }
    
    public static long getStartTime(CompositeData compositeData) {
        return LazyCompositeData.getLong(compositeData, "startTime");
    }
    
    public static long getEndTime(CompositeData compositeData) {
        return LazyCompositeData.getLong(compositeData, "endTime");
    }
    
    public static Map<String, MemoryUsage> getMemoryUsageBeforeGc(CompositeData compositeData) {
        try {
            return cast(GcInfoCompositeData.memoryUsageMapType.toJavaTypeData((Object)compositeData.get("memoryUsageBeforeGc")));
        }
        catch (InvalidObjectException | OpenDataException detailMessage) {
            throw new AssertionError(detailMessage);
        }
    }
    
    public static Map<String, MemoryUsage> cast(Object o) {
        return (Map)o;
    }
    
    public static Map<String, MemoryUsage> getMemoryUsageAfterGc(CompositeData compositeData) {
        try {
            return cast(GcInfoCompositeData.memoryUsageMapType.toJavaTypeData((Object)compositeData.get("memoryUsageAfterGc")));
        }
        catch (InvalidObjectException | OpenDataException detailMessage) {
            throw new AssertionError(detailMessage);
        }
    }
    
    public static void validateCompositeData(CompositeData compositeData) {
        if (compositeData == null) {
            throw new NullPointerException("Null CompositeData");
        }
        if (!LazyCompositeData.isTypeMatched(getBaseGcInfoCompositeType(), compositeData.getCompositeType())) {
            throw new IllegalArgumentException("Unexpected composite type for GcInfo");
        }
        return;
    }
    
    static synchronized CompositeType getBaseGcInfoCompositeType() {
        if (GcInfoCompositeData.baseGcInfoCompositeType == null) {
            try {
                GcInfoCompositeData.baseGcInfoCompositeType = new CompositeType("sun.management.BaseGcInfoCompositeType", "CompositeType for Base GcInfo", getBaseGcInfoItemNames(), getBaseGcInfoItemNames(), getBaseGcInfoItemTypes());
            }
            catch (OpenDataException ex) {
                throw Util.newException(ex);
            }
        }
        return GcInfoCompositeData.baseGcInfoCompositeType;
    }
    
    static {
        baseGcInfoItemNames = new String[] { "id", "startTime", "endTime", "duration", "memoryUsageBeforeGc", "memoryUsageAfterGc" };
        try {
            GcInfoCompositeData.memoryUsageMapType = MappedMXBeanType.getMappedType(GcInfo.class.getMethod("getMemoryUsageBeforeGc", new Class[0]).getGenericReturnType());
        }
        catch (NoSuchMethodException | OpenDataException detailMessage) {
            throw new AssertionError(detailMessage);
        }
        GcInfoCompositeData.baseGcInfoItemTypes = null;
        GcInfoCompositeData.baseGcInfoCompositeType = null;
    }
}
